@@ -16,7 +16,6 @@ class Gem::Commands::InstallCommand < Gem::Command
     defaults = Gem::DependencyInstaller::DEFAULT_OPTIONS.merge({
       :generate_rdoc => true,
       :generate_ri   => true,
-      :install_dir => Gem.dir,
       :format_executable => false,
       :test => false,
       :version => Gem::Requirement.default,
@@ -39,6 +38,19 @@ class Gem::Commands::InstallCommand < Gem::Command
     "--no-test --install-dir #{Gem.dir}"
   end
 
+  def description # :nodoc:
+    <<-EOF
+The install command installs local or remote gem into a gem repository.
+
+For gems with executables ruby installs a wrapper file into the executable
+directory by deault.  This can be overridden with the --no-wrappers option.
+The wrapper allows you to choose among alternate gem versions using _version_.
+
+For example `rake _0.7.3_ --version` will run rake version 0.7.3 if a newer
+version is also installed.
+    EOF
+  end
+
   def usage # :nodoc:
     "#{program_name} GEMNAME [GEMNAME ...] [options] -- --build-flags"
   end
@@ -51,7 +63,7 @@ class Gem::Commands::InstallCommand < Gem::Command
 
     installed_gems = []
 
-    ENV['GEM_PATH'] = options[:install_dir] # HACK what does this do?
+    ENV.delete 'GEM_PATH' if options[:install_dir].nil? and RUBY_VERSION > '1.9'
 
     install_options = {
       :env_shebang => options[:env_shebang],
@@ -63,7 +75,9 @@ class Gem::Commands::InstallCommand < Gem::Command
       :security_policy => options[:security_policy],
       :wrappers => options[:wrappers],
       :bin_dir => options[:bin_dir],
-      :skip_dependencies => options[:skip_dependencies]
+      :development => options[:development],
+      :skip_dependencies => options[:skip_dependencies],
+      :default_source_index => options[:default_source_index]
     }
 
     exit_code = 0
@@ -107,6 +121,8 @@ class Gem::Commands::InstallCommand < Gem::Command
       installed_gems.each do |gem|
         Gem::DocManager.new(gem, options[:rdoc_args]).generate_ri
       end
+
+      Gem::DocManager.update_ri_cache
     end
 
     if options[:generate_rdoc] then
