@@ -90,10 +90,28 @@ module Gem::InstallUpdateOptions
       options[:format_executable] = value
     end
 
+    add_option(:"Install/Update",       '--[no-]user-install',
+               'Install in user\'s home directory instead',
+               'of GEM_HOME. Defaults to using home directory',
+               'only if GEM_HOME is not writable.') do |value, options|
+      options[:user_install] = value
+    end
+
+    add_option(:"Install/Update", "--development",
+                "Install any additional development",
+                "dependencies") do |value, options|
+      options[:development] = true
+    end
+
     add_option(:"Install/Update", '-S', '--skip-dependencies DEPENDENCIES',
                'Skip the given dependencies') do |value, options|
       options[:skip_dependencies] = value.split(' ')
     end
+
+    add_option(:"Install/Update", '--default-source-index',
+                'Use the default source-index') do |value, options|
+      options[:default_source_index] = true
+    end
   end
 
   # Default options for the gem install command.
